import string
import random
from datetime import datetime, timedelta
from typing import Optional, Tuple, Dict
import logging
from .database import get_cursor
from .email_service import EmailService
from .config import CONFIG

logger = logging.getLogger(__name__)

class AuthService:
    def __init__(self, test_mode: bool = False):
        self.max_attempts = 3
        self.rate_limit_window = 3600  # 1 hour in seconds
        self.rate_limit_max = 5  # Maximum requests per hour
        self.email_service = EmailService(test_mode=test_mode)

    def generate_code(self, email: str, code_length: int = 6) -> Tuple[bool, str]:
        """Generate and store a verification code"""
        if not self._check_rate_limit(email, "generate_code"):
            logger.warning(f"Rate limit exceeded for {email}")
            return False, "Rate limit exceeded"

        try:
            # Generate a simple code
            code = ''.join(random.choices(string.ascii_uppercase + string.digits, k=code_length))
            expires_at = datetime.now() + timedelta(minutes=15)

            with get_cursor() as cursor:
                # Clean up any expired codes
                cursor.execute("""
                    DELETE FROM verification_codes 
                    WHERE email = %s AND expires_at < NOW()
                """, (email,))

                # Insert new code
                cursor.execute("""
                    INSERT INTO verification_codes (email, code, expires_at, attempts)
                    VALUES (%s, %s, %s, 0)
                    RETURNING id
                """, (email, code, expires_at))

                result = cursor.fetchone()
                if result:
                    code_id = result['id']
                    email_sent = self.email_service.send_verification_code(email, code)
                    if email_sent:
                        logger.info(f"Generated code for {email} (ID: {code_id})")
                        return True, code
                    logger.error(f"Failed to send email to {email}")
                return False, ""

        except Exception as e:
            logger.error(f"Code generation error: {e}")
            return False, ""

    def verify_code(self, email: str, code: str, days: int = 0) -> bool:
        """Verify a code with time limit in days"""
        if not self._check_rate_limit(email, "verify_code"):
            logger.warning(f"Rate limit exceeded for {email}")
            return False

        try:
            with get_cursor() as cursor:
                # Get the exact code for this email
                cursor.execute("""
                    SELECT id, code FROM verification_codes
                    WHERE email = %s 
                    AND code = %s
                    AND used = FALSE
                    AND attempts < %s
                    AND expires_at > NOW()
                    AND created_at > NOW() - INTERVAL '%s days'
                    ORDER BY created_at DESC
                    LIMIT 1
                """, (email, code, self.max_attempts, max(days, 0)))

                result = cursor.fetchone()
                if result:
                    # Mark code as used on successful verification
                    cursor.execute("""
                        UPDATE verification_codes
                        SET used = TRUE
                        WHERE id = %s
                    """, (result['id'],))
                    logger.info(f"Code {code} verified successfully for {email}")
                    return True

                # Increment attempts on failed verification
                cursor.execute("""
                    UPDATE verification_codes
                    SET attempts = attempts + 1
                    WHERE email = %s 
                    AND code = %s
                    AND used = FALSE
                    AND expires_at > NOW()
                """, (email, code))
                logger.warning(f"Failed verification attempt for {email}")
                return False

        except Exception as e:
            logger.error(f"Database error during verification: {e}")
            return False

    def _check_rate_limit(self, email: str, endpoint: str) -> bool:
        """Check if the email has exceeded rate limits for the endpoint"""
        try:
            with get_cursor() as cursor:
                # Clean up old rate limit entries
                cursor.execute("""
                    DELETE FROM rate_limits 
                    WHERE window_start < NOW() - INTERVAL '1 hour'
                """)

                # Get or create rate limit entry
                cursor.execute("""
                    INSERT INTO rate_limits (email, endpoint, request_count, window_start)
                    VALUES (%s, %s, 1, NOW())
                    ON CONFLICT (email, endpoint)
                    DO UPDATE SET 
                        request_count = rate_limits.request_count + 1
                    WHERE 
                        rate_limits.window_start > NOW() - INTERVAL '1 hour'
                    RETURNING request_count
                """, (email, endpoint))

                result = cursor.fetchone()
                return bool(result and result.get('request_count', 0) <= self.rate_limit_max)
        except Exception as e:
            logger.error(f"Rate limit check error: {e}")
            return False

    def resign_codes(self, email: str) -> bool:
        """Cancel all active verification codes for an email address."""
        try:
            with get_cursor() as cursor:
                cursor.execute("""
                    UPDATE verification_codes
                    SET used = TRUE
                    WHERE email = %s AND used = FALSE
                """, (email,))

                affected = cursor.rowcount > 0
                if affected:
                    logger.info(f"Resigned all codes for {email}")
                return affected

        except Exception as e:
            logger.error(f"Code resignation error: {e}")
            return False

    def quick_check(self, code: str) -> bool:
        """Quickly check if a verification code is still valid."""
        try:
            with get_cursor() as cursor:
                cursor.execute("""
                    SELECT EXISTS(
                        SELECT 1 FROM verification_codes
                        WHERE code = %s 
                        AND NOT used 
                        AND expires_at > NOW()
                        AND attempts < %s
                    )
                """, (code, self.max_attempts))

                result = cursor.fetchone()
                return bool(result and result['exists'])

        except Exception as e:
            logger.error(f"Quick check error: {e}")
            return False