from fastapi import FastAPI, HTTPException, Request
from fastapi.middleware.cors import CORSMiddleware
from pydantic import BaseModel, EmailStr
from typing import Optional, Dict
from .auth_service import AuthService
from .subscription_service import SubscriptionService
from .database import init_db
from .rate_limiter import create_rate_limiter
import logging
import os

# Configure logging
logging.basicConfig(
    level=logging.DEBUG,
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
)
logger = logging.getLogger(__name__)

app = FastAPI(title="RedXAuth API")

# Add CORS middleware
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

# Initialize services
auth_service = AuthService()
subscription_service = SubscriptionService()

# Add rate limiting middleware
app.middleware("http")(create_rate_limiter())

class CodeRequest(BaseModel):
    email: EmailStr
    code_length: int = 6

class CodeVerification(BaseModel):
    email: EmailStr
    code: str
    days: int = 0

class SubscriptionRequest(BaseModel):
    email: EmailStr
    plan_type: str
    duration_days: int

@app.on_event("startup")
async def startup():
    try:
        logger.info("Initializing database...")
        init_db()
        logger.info("Database initialization completed")
    except Exception as e:
        logger.error(f"Failed to initialize database: {e}")
        raise

@app.get("/health")
async def health_check():
    return {"status": "healthy", "database_url": os.environ.get('DATABASE_URL') is not None}

@app.post("/generate-code")
async def generate_code(request: CodeRequest) -> Dict:
    try:
        success, code = auth_service.generate_code(request.email, request.code_length)
        if not success:
            raise HTTPException(status_code=500, detail="Failed to send verification code")
        return {"message": "Verification code sent successfully"}
    except Exception as e:
        logger.error(f"Error generating code: {e}")
        raise HTTPException(status_code=500, detail=str(e))

@app.post("/verify-code")
async def verify_code(verification: CodeVerification) -> Dict:
    try:
        if auth_service.verify_code(verification.email, verification.code, verification.days):
            return {"verified": True}
        raise HTTPException(status_code=400, detail="Invalid or expired code")
    except Exception as e:
        logger.error(f"Error verifying code: {e}")
        raise HTTPException(status_code=500, detail=str(e))

@app.post("/quick-check/{code}")
async def quick_check(code: str) -> Dict:
    try:
        return {"valid": auth_service.quick_check(code)}
    except Exception as e:
        logger.error(f"Error in quick check: {e}")
        raise HTTPException(status_code=500, detail=str(e))

@app.post("/create-subscription")
async def create_subscription(request: SubscriptionRequest) -> Dict:
    try:
        subscription = subscription_service.create_subscription(
            request.email,
            request.plan_type,
            request.duration_days
        )
        return subscription
    except Exception as e:
        logger.error(f"Error creating subscription: {e}")
        raise HTTPException(status_code=500, detail=str(e))

@app.get("/check-subscription/{email}")
async def check_subscription(email: EmailStr) -> Dict:
    try:
        return subscription_service.check_subscription(email)
    except Exception as e:
        logger.error(f"Error checking subscription: {e}")
        raise HTTPException(status_code=500, detail=str(e))

@app.post("/resign-codes/{email}")
async def resign_codes(email: EmailStr) -> Dict:
    try:
        if auth_service.resign_codes(email):
            return {"message": "All verification codes cancelled successfully"}
        raise HTTPException(status_code=404, detail="No active codes found")
    except Exception as e:
        logger.error(f"Error resigning codes: {e}")
        raise HTTPException(status_code=500, detail=str(e))

@app.post("/resign-subscription/{email}")
async def resign_subscription(email: EmailStr) -> Dict:
    try:
        if subscription_service.resign_subscription(email):
            return {"message": "Subscription cancelled successfully"}
        raise HTTPException(status_code=404, detail="No active subscription found")
    except Exception as e:
        logger.error(f"Error resigning subscription: {e}")
        raise HTTPException(status_code=500, detail=str(e))

if __name__ == "__main__":
    import uvicorn
    uvicorn.run(app, host="0.0.0.0", port=8000)