from fastapi import FastAPI, Request, HTTPException
from fastapi.responses import HTMLResponse
from fastapi.templating import Jinja2Templates
from fastapi.staticfiles import StaticFiles
import os
import sys
from pathlib import Path
import logging
import traceback
from datetime import datetime, timedelta
from typing import Optional

# Set up detailed logging
logging.basicConfig(
    level=logging.DEBUG,
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
)
logger = logging.getLogger(__name__)

# Add parent directory to Python path to resolve imports
parent_dir = str(Path(__file__).resolve().parent.parent)
if parent_dir not in sys.path:
    sys.path.append(parent_dir)
    logger.debug(f"Added {parent_dir} to Python path")

try:
    from auth_api.code_generator import RedXCode
    from auth_api.subscription_service import SubscriptionService
    from auth_api.email_service import EmailService
    import RedXSecurePayments
    logger.info("Successfully imported required modules")
except Exception as e:
    logger.error(f"Failed to import modules: {str(e)}\n{traceback.format_exc()}")
    raise

app = FastAPI()

# Set up templates directory
try:
    base_dir = Path(__file__).resolve().parent
    templates_dir = base_dir / "templates"
    templates_dir.mkdir(exist_ok=True)
    templates = Jinja2Templates(directory=str(templates_dir))
    logger.info(f"Templates directory set up at {templates_dir}")
except Exception as e:
    logger.error(f"Failed to set up templates: {str(e)}\n{traceback.format_exc()}")
    raise

# Initialize services with proper error handling
try:
    code_generator = RedXCode()
    subscription_service = SubscriptionService()
    email_service = EmailService()
    logger.info("Services initialized successfully")
except Exception as e:
    logger.error(f"Failed to initialize services: {str(e)}\n{traceback.format_exc()}")
    raise

@app.on_event("startup")
async def startup_event():
    logger.info("FastAPI application starting up")

@app.get("/", response_class=HTMLResponse)
async def payment_test_page(request: Request):
    try:
        logger.debug("Attempting to render payment test page")
        return templates.TemplateResponse(
            "payment_test.html", 
            {"request": request}
        )
    except Exception as e:
        logger.error(f"Failed to render template: {str(e)}\n{traceback.format_exc()}")
        raise HTTPException(status_code=500, detail=str(e))

@app.post("/process_payment/{package_type}")
async def process_payment(package_type: str, email: str):
    try:
        logger.info(f"Processing payment for package: {package_type}, email: {email}")

        # Verify email is provided
        if not email:
            return {"success": False, "error": "Email is required"}

        # Set payment amount based on package type
        amounts = {
            "auth_code": 5.00,
            "subscription": 10.00,
            "complete": 15.00
        }
        amount = amounts.get(package_type)
        if amount is None:
            return {"success": False, "error": "Invalid package type"}

        try:
            # Process payment using RedXSecurePayments
            payment_result = RedXSecurePayments.RedXSecureBuy(
                bank_name=f"RedXAuth {package_type.title()} Package",
                currency_type="usd",
                currency_amount=amount,
                enable_multiple_payments=False,
                enable_custom_amount=False,
                enable_payment_history=True
            )
        except Exception as e:
            logger.error(f"Payment processing failed: {str(e)}")
            return {"success": False, "error": "Payment processing failed"}

        if not payment_result.get('success'):
            logger.error(f"Payment failed: {payment_result.get('error')}")
            return {"success": False, "error": payment_result.get('error', 'Payment failed')}

        # Check payment status
        try:
            status = RedXSecurePayments.get_payment_status(payment_result['session_id'])
            if not status.get('success'):
                logger.error("Payment verification failed")
                return {"success": False, "error": "Payment verification failed"}
        except Exception as e:
            logger.error(f"Payment status check failed: {str(e)}")
            return {"success": False, "error": "Payment verification failed"}

        # Initialize response
        result = {
            "success": True,
            "payment_url": payment_result['payment_url'],
            "session_id": payment_result['session_id']
        }

        # Handle code generation if needed
        if package_type in ["auth_code", "complete"]:
            try:
                code_result = code_generator.Generate(
                    email=email,
                    length=8,
                    expiry_minutes=15
                )

                if not code_result:
                    logger.error("Failed to generate verification code")
                    return {"success": False, "error": "Failed to generate verification code"}

                code, created_at, expires_at = code_result
                result["code"] = code
                result["code_expires"] = expires_at

                # Send verification email
                if not email_service.send_verification_code(email, code):
                    logger.error(f"Failed to send verification code email to {email}")
                    return {"success": False, "error": "Failed to send verification code"}

                logger.info(f"Generated and sent auth code to {email}")
            except Exception as e:
                logger.error(f"Code generation/email failed: {str(e)}")
                return {"success": False, "error": "Failed to process verification code"}

        # Handle subscription if needed
        if package_type in ["subscription", "complete"]:
            try:
                subscription = subscription_service.create_subscription(
                    email=email,
                    plan_type="Premium",
                    duration_days=30,
                    custom_message="Thank you for subscribing to RedXAuth Premium!"
                )

                if not subscription:
                    logger.error("Failed to create subscription")
                    return {"success": False, "error": "Failed to create subscription"}

                result["subscription_expires"] = subscription['expiration_time']
                logger.info(f"Created subscription for {email}")
            except Exception as e:
                logger.error(f"Subscription creation failed: {str(e)}")
                return {"success": False, "error": "Failed to create subscription"}

        logger.info("Payment processing completed successfully")
        return result

    except Exception as e:
        logger.error(f"Error processing payment: {str(e)}\n{traceback.format_exc()}")
        raise HTTPException(status_code=500, detail=str(e))

if __name__ == "__main__":
    import uvicorn
    logger.info("Starting FastAPI application directly")
    uvicorn.run(app, host="0.0.0.0", port=8001, log_level="debug")