from fastapi import Request, HTTPException
from typing import Optional, Dict, Callable
from datetime import datetime, timedelta
from .database import get_cursor
import logging

logger = logging.getLogger(__name__)

class RateLimiter:
    def __init__(self, requests_per_minute: int = 30):
        self.requests_per_minute = requests_per_minute
        self.endpoints_limits: Dict[str, int] = {
            "/generate-code": 5,  # 5 requests per minute
            "/verify-code": 10,   # 10 attempts per minute
            "/create-subscription": 3,  # 3 subscription creations per minute
            "/check-subscription": 10,  # 10 checks per minute
            "/resign-codes": 5,    # 5 resignations per minute
            "/resign-subscription": 5  # 5 subscription cancellations per minute
        }

    async def _get_client_email(self, request: Request) -> str:
        """Extract email from request - either from params, body or use IP as fallback"""
        try:
            # Try to get email from query parameters
            email = request.query_params.get('email')
            if email:
                return email

            # Try to get email from request body
            body = await request.json()
            if isinstance(body, dict) and 'email' in body:
                return body['email']

        except:
            pass

        # Fallback to IP address if email not found
        return request.client.host

    async def check_rate_limit(self, request: Request) -> None:
        """Check if the request should be rate limited"""
        try:
            path = request.url.path
            identifier = await self._get_client_email(request)
            limit = self.endpoints_limits.get(path, self.requests_per_minute)

            with get_cursor() as cursor:
                # Clean up old rate limit entries
                cursor.execute("""
                    DELETE FROM rate_limits 
                    WHERE window_start < NOW() - INTERVAL '1 minute'
                """)

                # Get or create rate limit entry
                cursor.execute("""
                    INSERT INTO rate_limits (email, endpoint, request_count, window_start)
                    VALUES (%s, %s, 1, NOW())
                    ON CONFLICT (email, endpoint)
                    DO UPDATE SET 
                        request_count = rate_limits.request_count + 1
                    WHERE 
                        rate_limits.window_start > NOW() - INTERVAL '1 minute'
                    RETURNING request_count
                """, (identifier, path))

                result = cursor.fetchone()
                if result and result['request_count'] > limit:
                    logger.warning(f"Rate limit exceeded for {identifier} on {path}")
                    raise HTTPException(
                        status_code=429,
                        detail=f"Rate limit exceeded. Try again in 1 minute."
                    )

        except HTTPException:
            raise
        except Exception as e:
            logger.error(f"Rate limit check error: {e}")
            # Allow request to proceed on error
            pass

def create_rate_limiter(requests_per_minute: int = 30) -> Callable:
    """Create a FastAPI middleware for rate limiting"""
    limiter = RateLimiter(requests_per_minute)

    async def rate_limit_middleware(request: Request, call_next):
        await limiter.check_rate_limit(request)
        return await call_next(request)

    return rate_limit_middleware
