from datetime import datetime, timedelta
import threading
import time
from typing import Optional
from .database import get_cursor
from .email_service import EmailService
import logging

logger = logging.getLogger(__name__)

class RedXSubscription:
    def __init__(self):
        self.email_service = EmailService()  # Changed from RedXVerifyEmail to EmailService
        self._stop_checker = False
        self._checker_thread = None
        self.start_subscription_checker()

    def Create(self, email: str, expiration_date: str, subscription_name: str, subscription_type: str = "") -> bool:
        """Create a new subscription
        Args:
            email: User's email
            expiration_date: Expiration date in MM/DD/YYYY/HH:MMam/pm format
            subscription_name: Name of the subscription
            subscription_type: Type of subscription (optional)
        """
        try:
            # Parse the expiration date to verify format
            try:
                datetime.strptime(expiration_date.lower(), "%m/%d/%Y/%I:%M%p")
            except ValueError:
                logger.error(f"Invalid expiration date format: {expiration_date}")
                return False

            with get_cursor() as cursor:
                # Delete any existing subscription for this email
                cursor.execute("""
                    DELETE FROM redx_subscriptions 
                    WHERE email = %s
                """, (email,))

                # Create new subscription
                cursor.execute("""
                    INSERT INTO redx_subscriptions 
                    (email, start_time, expiration_time, subscription_name, subscription_type)
                    VALUES (%s, %s, %s, %s, %s)
                    RETURNING id
                """, (
                    email,
                    datetime.now().strftime("%m/%d/%Y/%I:%M%p").lower(),
                    expiration_date.lower(),
                    subscription_name,
                    subscription_type
                ))

                result = cursor.fetchone()
                if result:
                    logger.info(f"Created subscription for {email}, expires {expiration_date}")
                    return True
                return False

        except Exception as e:
            logger.error(f"Error creating subscription: {e}")
            return False

    def ActiveCheck(self, email: str) -> bool:
        """Check if user has an active subscription"""
        try:
            current_time = datetime.now().strftime("%m/%d/%Y/%I:%M%p").lower()

            with get_cursor() as cursor:
                # Check for active subscription
                cursor.execute("""
                    SELECT EXISTS(
                        SELECT 1 FROM redx_subscriptions
                        WHERE email = %s 
                        AND expiration_time > %s
                    )
                """, (email, current_time))

                result = cursor.fetchone()
                return result['exists'] if result else False

        except Exception as e:
            logger.error(f"Error checking subscription: {e}")
            return False

    def _check_subscriptions(self):
        """Background thread to check and delete expired subscriptions"""
        while not self._stop_checker:
            try:
                current_time = datetime.now()
                current_time_str = current_time.strftime("%m/%d/%Y/%I:%M%p").lower()

                with get_cursor() as cursor:
                    # Delete expired subscriptions
                    cursor.execute("""
                        DELETE FROM redx_subscriptions
                        WHERE expiration_time <= %s
                        RETURNING email, subscription_name
                    """, (current_time_str,))

                    for expired in cursor.fetchall():
                        logger.info(f"Subscription expired and deleted: {expired['email']} - {expired['subscription_name']}")

            except Exception as e:
                logger.error(f"Error in subscription checker: {e}")

            # Sleep for 1 hour before next check
            time.sleep(3600)

    def start_subscription_checker(self):
        """Start the background subscription checker thread"""
        if not self._checker_thread or not self._checker_thread.is_alive():
            self._stop_checker = False
            self._checker_thread = threading.Thread(target=self._check_subscriptions)
            self._checker_thread.daemon = True
            self._checker_thread.start()

    def stop_subscription_checker(self):
        """Stop the background subscription checker thread"""
        self._stop_checker = True
        if self._checker_thread:
            self._checker_thread.join(timeout=1)

    def RemoveData(self, email: str) -> bool:
        """Remove all subscription data for a given email address"""
        try:
            with get_cursor() as cursor:
                cursor.execute("""
                    DELETE FROM redx_subscriptions
                    WHERE email = %s
                """, (email,))
                deleted = cursor.rowcount > 0
                if deleted:
                    logger.info(f"Removed all subscription data for {email}")
                return deleted
        except Exception as e:
            logger.error(f"Error removing subscription data: {e}")
            return False