import tkinter as tk
from tkinter import ttk, messagebox
import RedXSecurePayments
from .auth_service import AuthService
from .subscription_service import SubscriptionService
from .code_generator import RedXCode
from datetime import datetime, timedelta

class TestUI:
    def __init__(self):
        self.window = tk.Tk()
        self.window.title("RedXAuth Test Interface")
        self.window.geometry("400x300")

        self.auth_service = AuthService()
        self.subscription_service = SubscriptionService()
        self.code_generator = RedXCode()

        # Email input
        ttk.Label(self.window, text="Email:").pack(pady=5)
        self.email_var = tk.StringVar()
        ttk.Entry(self.window, textvariable=self.email_var).pack(pady=5)

        # Auth Code Button
        ttk.Label(self.window, text="Authentication Code - $5").pack(pady=5)
        ttk.Button(self.window, text="Buy Auth Code", 
                  command=self.buy_auth_code).pack(pady=5)

        # Subscription Button
        ttk.Label(self.window, text="Monthly Subscription - $10").pack(pady=5)
        ttk.Button(self.window, text="Buy Subscription", 
                  command=self.buy_subscription).pack(pady=5)

        # Complete Package Button
        ttk.Label(self.window, text="Complete Package - $15").pack(pady=5)
        ttk.Label(self.window, text="Includes:\n- Authentication Code\n- 30-day Subscription").pack(pady=5)
        ttk.Button(self.window, text="Buy Complete Package", 
                  command=self.buy_complete_package).pack(pady=5)

    def buy_auth_code(self):
        email = self.email_var.get()
        if not email:
            messagebox.showerror("Error", "Please enter an email address")
            return

        try:
            payment_result = RedXSecurePayments.RedXSecureBuy(
                bank_name="RedXAuth Code",
                currency_type="usd",
                currency_amount=5.00,  # $5.00
                enable_multiple_payments=False,
                enable_custom_amount=False,
                enable_payment_history=True
            )

            if payment_result['success']:
                # Check payment status
                status = RedXSecurePayments.get_payment_status(payment_result['session_id'])

                if status['success'] and status['status'] == 'completed':
                    code_result = self.code_generator.Generate(
                        email=email,
                        length=8,
                        expiry_minutes=15
                    )
                    if code_result:
                        code, created_at, expires_at = code_result
                        messagebox.showinfo("Success", 
                            f"Payment processed!\nPayment URL: {payment_result['payment_url']}\nAuth code sent to your email!")
                        print(f"Generated Code: {code}")
                        print(f"Code expires: {expires_at}")
                        print(f"Payment session ID: {payment_result['session_id']}")
                    else:
                        messagebox.showerror("Error", 
                            "Payment successful but failed to generate code")
                else:
                    messagebox.showerror("Error", "Payment verification failed")
            else:
                messagebox.showerror("Error", f"Payment failed: {payment_result.get('error', 'Unknown error')}")

        except Exception as e:
            messagebox.showerror("Error", str(e))

    def buy_subscription(self):
        email = self.email_var.get()
        if not email:
            messagebox.showerror("Error", "Please enter an email address")
            return

        try:
            payment_result = RedXSecurePayments.RedXSecureBuy(
                bank_name="RedXAuth Subscription",
                currency_type="usd",
                currency_amount=10.00,  # $10.00
                enable_multiple_payments=False,
                enable_custom_amount=False,
                enable_payment_history=True
            )

            if payment_result['success']:
                # Check payment status
                status = RedXSecurePayments.get_payment_status(payment_result['session_id'])

                if status['success'] and status['status'] == 'completed':
                    expiry_date = (datetime.now() + timedelta(days=30)).strftime("%m/%d/%Y/%I:%M%p").lower()
                    success = self.subscription_service.create_subscription(
                        email=email,
                        plan_type="monthly",
                        duration_days=30
                    )
                    if success:
                        messagebox.showinfo("Success", 
                            f"Payment processed!\nPayment URL: {payment_result['payment_url']}\nSubscription activated!")
                        print(f"Payment session ID: {payment_result['session_id']}")
                    else:
                        messagebox.showerror("Error", 
                            "Payment successful but failed to activate subscription")
                else:
                    messagebox.showerror("Error", "Payment verification failed")
            else:
                messagebox.showerror("Error", f"Payment failed: {payment_result.get('error', 'Unknown error')}")

        except Exception as e:
            messagebox.showerror("Error", str(e))

    def buy_complete_package(self):
        email = self.email_var.get()
        if not email:
            messagebox.showerror("Error", "Please enter an email address")
            return

        try:
            payment_result = RedXSecurePayments.RedXSecureBuy(
                bank_name="RedXAuth Complete Package",
                currency_type="usd",
                currency_amount=15.00,  # $15.00
                enable_multiple_payments=False,
                enable_custom_amount=False,
                enable_payment_history=True
            )

            if payment_result['success']:
                # Check payment status
                status = RedXSecurePayments.get_payment_status(payment_result['session_id'])

                if status['success'] and status['status'] == 'completed':
                    # Generate auth code
                    code_result = self.code_generator.Generate(
                        email=email,
                        length=8,
                        expiry_minutes=15
                    )

                    # Create subscription
                    expiry_date = (datetime.now() + timedelta(days=30)).strftime("%m/%d/%Y/%I:%M%p").lower()
                    subscription = self.subscription_service.create_subscription(
                        email=email,
                        plan_type="Premium",
                        duration_days=30
                    )

                    if code_result and subscription:
                        code, created_at, expires_at = code_result
                        messagebox.showinfo("Success", 
                            f"Payment processed successfully!\nPayment URL: {payment_result['payment_url']}\nVerification code and subscription details sent to your email.")
                        print(f"Generated Code: {code}")
                        print(f"Code expires: {expires_at}")
                        print(f"Subscription expires: {expiry_date}")
                        print(f"Payment session ID: {payment_result['session_id']}")
                    else:
                        messagebox.showerror("Error", 
                            "Payment successful but failed to generate products")
                else:
                    messagebox.showerror("Error", "Payment verification failed")
            else:
                messagebox.showerror("Error", f"Payment failed: {payment_result.get('error', 'Unknown error')}")

        except Exception as e:
            messagebox.showerror("Error", str(e))

    def run(self):
        self.window.mainloop()

if __name__ == "__main__":
    ui = TestUI()
    ui.run()